@@ -1,3 +1,4 @@
+/* -*- mode:C; indent-tabs-mode:nill; tab-width:8; c-basic-offset:8 -*- */
 /*
  * written by J. Marcin Gorycki <marcin.gorycki@intel.com>
  *
@@ -26,9 +27,10 @@
 
 
 BlockOps::BlockOps() :
-  blocknr (0),
-  rot (0),
-  color (0)
+     useTarget (false),
+     blocknr (0),
+     rot (0),
+     color (0)
 {
 	field = new Block*[COLUMNS];
 	
@@ -175,6 +177,46 @@ BlockOps::moveBlockDown()
 	return fallen;
 }
 
+void 
+BlockOps::clearTarget ()
+{
+	for (int x = 0; x < COLUMNS; ++x)
+		for (int y = 0; y < LINES; ++y)
+			if (field[x][y].what == TARGET)
+				field[x][y].what = EMPTY;
+}
+
+// The target is the set of blocks which the currently falling block
+// will occupy when it lands. It is an aid for beginners.
+void
+BlockOps::generateTarget ()
+{
+	if (!useTarget)
+		return;
+
+	clearTarget ();
+
+	// FIXME: Check that this is actually guaranteed
+	// to terminate (i.e. posx, posy, blocknr and rot 
+	// are guaranteed to be valid).
+	int n = 0;
+	do {
+		n++;
+	} while (blockOkHere (posx, posy + n, blocknr, rot));
+	n--;
+
+	// Mark the relevant places.
+	putBlockInField (posx, posy + n, blocknr, rot, TARGET);
+}
+
+void 
+BlockOps::setUseTarget (bool use)
+{
+	useTarget = use;
+	if (!useTarget)
+		clearTarget ();
+}
+
 int
 BlockOps::dropBlock()
 {
@@ -303,26 +345,51 @@ BlockOps::emptyField(void)
 }
 
 void
-BlockOps::putBlockInField(bool erase)
+BlockOps::putBlockInField (int bx, int by, int block, int rotation,
+			   SlotType fill)
 {
 	for (int x = 0; x < 4; ++x)
 	{
 		for (int y = 0; y < 4; ++y)
 		{
-			if (blockTable[blocknr][rot][x][y])
+			if (blockTable[block][rotation][x][y])
 			{
-				if (erase)
-				{
-					field[posx - 2 + x][y + posy].what = EMPTY;
-					field[posx - 2 + x][y + posy].color = 0;
-				}
+				int i = bx - 2 + x;
+				int j = y + by;
+
+				if ((fill == TARGET) && 
+				    (field[i][j].what != EMPTY))
+					return;
+
+				field[i][j].what = fill;
+				if ((fill == FALLING) || (fill == LAYING))
+					field[i][j].color = color;
 				else
-				{
-					field[posx - 2 + x][y + posy].what = FALLING;
-					field[posx - 2 + x][y + posy].color = color;
-				}
+					field[i][j].color = 0;
 			}
 		}
 	}
 }
 
+// This is now just a wrapper. I'm not sure which version should be
+// used in general: having the field keep track of the block worries
+// me, but I can't say it is definitely wrong.
+void
+BlockOps::putBlockInField (bool erase)
+{
+	if (erase)
+		putBlockInField (posx, posy, blocknr, rot, EMPTY);
+	else
+		putBlockInField (posx, posy, blocknr, rot, FALLING);
+}
+
+bool
+BlockOps::isFieldEmpty (void)
+{
+  for (int x = 0; x < COLUMNS; x++) {
+    if (field[x][LINES-1].what != EMPTY)
+      return false;
+  }
+
+  return true;
+}
